/*
STM to SCALA conversion
Ian Scott (ian@norcimo.com)
Version 2.2 (February 2006)
*/

/* You may freely copy, redistribute and modify this code for non-commercial purposes provided you acknowledge the original author and make freely available, under identical conditions and restrictions, all code thus produced  */

/*
Converts old fromat STM files to new format STM files
i.e. stmprg files to SCALA files
*/


#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <stdio.h>
#include "util.h"
#include "stm2scalaf.h"

#ifndef boolian
enum BOOL {FALSE,TRUE};
#endif

// #define DEBUG
//remove above comment for debug operations

int uns2s16bit(char highbyte, char lowbyte);
int convert(char*);
void errormess(char*, int);

int main(int argc, char* argv[])
{
int endfile, i;
SYNTYPE syn;

cout<<endl<<"STM to SCALA format conversion"<<endl;
cout<<"Version 2.2 February 2006"<<endl<<endl;

// Find the program name (without the path and .exe)
while(strchr(argv[0],'\\'))
	argv[0]=strchr(argv[0],'\\')+1;
if (strchr(argv[0],'.'))
	argv[0]=strtok(argv[0],".");

syn=syncheck(argc, argv); //check which syntax is in use

if (syn!=HELP) //Advise on how to get help
	cout<<"Type "<<argv[0]<<" help"<<" to get basic help"<<endl<<endl;
switch (syn)
	{
	case HELP: //stm2scala help or stm2scala ?
		{
		#ifdef DEBUG
		cout<<"Syntax help (HELP)"<<endl;
		cout<<"Convert nothing, just exit"<<endl;
		#endif
		cout<<endl<<argv[0]<<"    	            : Convert all available files 1 to 500"<<endl;
		cout<<argv[0]<<" <filelist>        : Convert all files in <filelist>"<<endl;
		cout<<argv[0]<<" <file> -          : Convert files from <file> onwards"<<endl;
		cout<<argv[0]<<" - <file>          : Convert all available files from 1 to <file>"<<endl;
		cout<<argv[0]<<" <file1> - <file2> : Convert all available files in the range <file1>		    		      to <file2>"<<endl;
		cout<<endl<<"Exiting..."<<endl<<endl;
		return 5;
		break;
		}

	case LIST: //stmscala <filelist>
		{
		#ifdef DEBUG
		cout<<"Syntax <filelist> (LIST)"<<endl;
		#endif
		for (endfile=1;endfile<argc;endfile++)
			{
			#ifdef DEBUG
			cout<<"Convert "<<argv[endfile]<<endl;
			#endif
			i=convert(argv[endfile]);
			if (i)
				{
				errormess(argv[endfile],i);
				exit(1);
				}
			}
		break;
		}
	case UPWARD: //stm2scala - <file>
		{
		#ifdef DEBUG
		cout<<"Syntax - <file> (UPWARD)"<<endl;
		#endif
		endfile=atoi(argv[2]);
		strcpy(argv[0],"0");
		while(nextfile(argv[0],endfile))
			{
			#ifdef DEBUG
			cout<<"Convert "<<argv[0]<<endl;
			#endif
			i=convert(argv[0]);
			if (i)
				{
				errormess(argv[0],i);
				exit(1);
				}
			}
		break;
		}

	case ALL://stm2scala
		{
		#ifdef DEBUG
		cout<<"Syntax stm2scala (ALL)"<<endl;
		#endif
		strcpy(argv[0],"0");
		while(nextfile(argv[0],500))
			{
			#ifdef DEBUG
			cout<<"Convert "<<argv[0]<<endl;
			#endif
			i=convert(argv[0]);
			if (i)
				{
				errormess(argv[0],i);
				exit(1);
				}
			}
		break;
		}

	case ONWARD: //stm2scala <file> -
		{
		#ifdef DEBUG
		cout<<"Syntax <file> - (ONWARD)"<<endl;
		#endif
		strcpy(argv[0],argv[1]);
		while(nextfile(argv[0],500))
			{
			#ifdef DEBUG
			cout<<"Convert "<<argv[0]<<endl;
			#endif
			i=convert(argv[0]);
			if (i)
				{
				errormess(argv[0],i);
				exit(1);
				}
			}
		break;
		}
	case RANGE: //stm2scala <file) - <file>
		{
		#ifdef DEBUG
		cout<<"Syntax <file> - <file> (RANGE)"<<endl;
		#endif
		endfile=atoi(argv[3]);
		strcpy(argv[0],argv[1]);
		while (nextfile(argv[0],endfile))
			{
			#ifdef DEBUG
			cout<<"Convert "<<argv[0]<<endl;
			#endif
			i=convert(argv[0]);
			if (i)
				{
				errormess(argv[0],i);
				exit(1);
				}
			}
		break;
		}
	} //end syntax switch. We've now converted all the files.

cout<<endl<<"All files converted."<<endl;
cout<<"Exiting.."<<endl<<endl;
return 0;
}

int convert(char* fnum)
/* This is the bit that does all the work converting. Basically legacy
code from around version 1.0 when the programme first worked converting
a single file, so it probably is a bit of a hack and messy */
{
ifstream oldparam,oldspec,oldtopog;
ofstream newparam,newspec,newtopog;
#ifdef DEBUG
ofstream debugstrm;
#endif
fstream temparam;
int tempi;
float tempf,temp2;
int i, convfile, pnts, lins, thislin, thispnt, convalue, specpnts, thisspecpnt;
char fname[255];
char lowbyte,highbyte;
BOOL backflag, specflag;


	strcpy(fname,"tp");
	strcat(fname,fnum);
	oldparam.open(fname,ios::in|ios::binary|ios::nocreate);
	if (!oldparam)
		{
		return 1;
		}
#ifdef DEBUG
	cout<<"Old param file open"<<endl;
#endif
	strcpy(fname,"tz");
	strcat(fname,fnum);
	oldtopog.open(fname,ios::in|ios::binary|ios::nocreate);
	if (!oldtopog)
		{
		return 2;
		}
#ifdef DEBUG
	cout<<"old topog file open"<<endl;
#endif
	strcpy(fname,"ts");
	strcat(fname,fnum);
	oldspec.open(fname,ios::in|ios::binary|ios::nocreate);
	if (!oldspec)
		{
		specflag=FALSE;
		}
	else
		{
		specflag=TRUE;
#ifdef DEBUG
		cout<<"Old spec file open"<<endl;
#endif
		}
	strcpy(fname,"tmp.par");
	temparam.open(fname,ios::in|ios::out|ios::binary);
	if (!temparam)
		{
		return 3;
		}
#ifdef DEBUG
	cout<<"temp file open"<<endl;
#endif
	oldparam.seekg(-1,ios::end);
	while(oldparam.tellg()!=ios::beg)
		{
		oldparam.get(lowbyte);
		temparam.put(lowbyte);
		oldparam.seekg(-2,ios::cur);
		}
	oldparam.seekg(0,ios::beg);
	oldparam.get(lowbyte);
	temparam.put(lowbyte);
#ifdef DEBUG
	cout<<"temp file written"<<endl;
#endif
	strcpy(fname,"m");
	strcat(fname,fnum);
	strcat(fname,"_ori.par");
	newparam.open(fname,ios::out);
	if (!newparam)
		{
		return 4;
		}
	strcpy(fname,"m");
	strcat(fname,fnum);
	strcat(fname,"_ori.tf0");
	newtopog.open(fname,ios::out|ios::binary);
	if (!newtopog)
		{
		return 5;
		}
#ifdef DEBUG
	cout<<"new topog file open"<<endl;
#endif
	if (specflag)
		{
		strcpy(fname,"m");
		strcat(fname,fnum);
		strcat(fname,"_ori.sf0");
		newspec.open(fname,ios::out|ios::binary);
		if (!newspec)
			{
			return 6;
			}
#ifdef DEBUG
		cout<<"new spec file open"<<endl;
#endif
		}
	temparam.seekg(-32,ios::end);
	temparam.read((char*) &tempi,4);
	pnts=tempi;
#ifdef DEBUG
	cout<<"points="<<pnts<<endl;
#endif
	temparam.seekg(-36,ios::end);
	temparam.read((char*) &tempi,4);
	lins=tempi;
#ifdef DEBUG
	cout<<"lines="<<lins<<endl;
#endif
	temparam.seekg(-312,ios::end);
	temparam.read((char*) &tempi,4);
	if (tempi)
		{
		backflag=FALSE;
		}
	else
		{
		backflag=TRUE;
		}
#ifdef DEBUG
	if (backflag)
		{
		cout<<"Backwards"<<endl;
		}
	else
		{
		cout<<"No backwards"<<endl;
		}
#endif

#ifdef DEBUG
	debugstrm.open("debug.txt");
	if (!debugstrm)
		{
		return 100;
		}
#endif

	for (thislin=0;thislin<lins;thislin++)
		{
		for (thispnt=0;thispnt<pnts;thispnt++)
			{
			oldtopog.get(highbyte);
			oldtopog.get(lowbyte);
#ifdef DEBUG
			if (thispnt==0)
			{
			i=highbyte;
			debugstrm<<i<<" ";
			i=lowbyte;
			debugstrm<<i<<" ";
			}
#endif
			convalue=uns2s16bit(highbyte,lowbyte);
#ifdef DEBUG
			if (thispnt==0)
			debugstrm<<convalue<<" ";
#endif
			convalue-=32768;
#ifdef DEBUG
			if (thispnt==0)
			debugstrm<<convalue<<" ";
#endif
			tempi=convalue&65280;
			tempi/=256;
			highbyte=tempi;
			lowbyte=convalue&255;
			newtopog.put(highbyte);
			newtopog.put(lowbyte);
#ifdef DEBUG
			if (thispnt==0)
			{
			i=highbyte;
			debugstrm<<i<<" ";
			i=lowbyte;
			debugstrm<<i<<endl;
			}
#endif
			}
		}
	newtopog.close();
#ifdef DEBUG
	cout<<"done forward topog"<<endl;
#endif
	if (backflag)
		{
		strcpy(fname,"m");
		strcat(fname,fnum);
		strcat(fname,"_ori.tb0");
		newtopog.open(fname,ios::out|ios::binary);
		if (!newtopog)
			{
			return 7;
			}
		for (thislin=0;thislin<lins;thislin++)
			{
			for (thispnt=0;thispnt<pnts;thispnt++)
				{
				oldtopog.get(highbyte);
				oldtopog.get(lowbyte);
				convalue=uns2s16bit(highbyte,lowbyte);
				convalue-=32768;
				tempi=convalue&65280;
				tempi/=256;
				highbyte=tempi;
				lowbyte=convalue&255;
				newtopog.put(highbyte);
				newtopog.put(lowbyte);
				}
			}
#ifdef DEBUG
		cout<<"Done backward topog file"<<endl;
#endif
		}
	if (specflag)
		{
		temparam.seekg(-168,ios::end);
		temparam.read((char*) &tempi, 4);
		specpnts=tempi;
#ifdef DEBUG
		cout<<"specpnts="<<specpnts<<endl;
#endif
		for (thisspecpnt=0;thisspecpnt<specpnts;thisspecpnt++)
			{
			for (thislin=0;thislin<lins;thislin++)
				{
				for (thispnt=0;thispnt<pnts;thispnt++)
					{
#ifdef DEBUG
					if (thispnt==0)
						cout<<"Line:"<<thislin<<" Point:"<<thispnt<<" SpecPoint:"<<thisspecpnt<<endl;
#endif
					tempi=2*(thislin*specpnts*pnts+thispnt*specpnts+thisspecpnt);
					oldspec.seekg(tempi,ios::beg);
					oldspec.get(highbyte);
					oldspec.get(lowbyte);
					convalue=uns2s16bit(highbyte,lowbyte);
					convalue-=32768;
					tempi=convalue&65280;
					tempi/=256;
					highbyte=tempi;
					lowbyte=convalue&255;
					newspec.put(highbyte);
					newspec.put(lowbyte);
					}
				}

			}
		newspec.close();
#ifdef DEBUG
		cout<<"Done forward spec file"<<endl;
#endif
		if (backflag)
			{
			strcpy(fname,"m");
			strcat(fname,fnum);
			strcat(fname,"_ori.sb0");
			newspec.open(fname,ios::out|ios::binary);
			if (!newspec)
				{
				return 8;
				}
			temparam.seekg(-184,ios::end);
			temparam.read((char*) &tempi, 4);
			specpnts=tempi;
			for (thisspecpnt=0;thisspecpnt<specpnts;thisspecpnt++)
				{
				for (thislin=0;thislin<lins;thislin++)
					{
					for (thispnt=0;thispnt<pnts;thispnt++)
						{
#ifdef DEBUG
					if (thispnt==0)
						cout<<"Line:"<<thislin<<" Point:"<<thispnt<<" SpecPoint:"<<thisspecpnt<<endl;

#endif
tempi=2*(thislin*specpnts*pnts+thispnt*specpnts+thisspecpnt+specpnts*pnts*lins);
						oldspec.seekg(tempi,ios::beg);
						oldspec.get(highbyte);
						oldspec.get(lowbyte);
						convalue=uns2s16bit(highbyte,lowbyte);
						convalue-=32768;
						tempi=convalue&65280;
						tempi/=256;
						highbyte=tempi;
						lowbyte=convalue&255;
						newspec.put(highbyte);
						newspec.put(lowbyte);
						}
					}

				}
			newspec.close();
#ifdef DEBUG
			cout<<"Done backward spec file"<<endl;
#endif
			}
		}

#ifdef DEBUG
debugstrm.close();
#endif

#ifdef DEBUG
	cout<<"Starting parameter file.."<<endl;
#endif
	newparam<<";"<<endl;
	newparam<<";            Omicron SPM Control."<<endl;
	newparam<<";          Parameter file for SPM data."<<endl;
	newparam<<";"<<endl<<endl;
	newparam<<"Format                           : 1"<<endl<<endl;
	newparam<<"Version                          : 4.0"<<endl;
	newparam<<"System                           : SCALA"<<endl<<endl;
	newparam<<";"<<endl;
	newparam<<"; User Information."<<endl;
	newparam<<";"<<endl<<endl;
	newparam<<"Date                             : ";
	oldparam.seekg(312,ios::beg);
	oldparam.read(fname, 20);
	newparam<<fname<<endl;
	newparam<<"User                             : ";
	oldparam.seekg(382,ios::beg);
	oldparam.read(fname, 20);
	newparam<<fname<<endl;
	newparam<<"Comment                          : ";
	oldparam.seekg(332,ios::beg);
	oldparam.read(fname,50);
	newparam<<fname<<endl;
	newparam<<";"<<endl;
	newparam<<"; Scanner Description."<<endl;
	newparam<<";"<<endl<<endl;
	newparam<<"Name                             : AFM_N.SCA"<<endl;
	newparam<<"VGAP Contact                     : SAMPLE"<<endl<<endl;
	newparam<<";"<<endl;
	newparam<<"; Scanner Area Description."<<endl;
	newparam<<";"<<endl<<endl;
	newparam<<"Field X Size in nm               : ";
	temparam.seekg(-16,ios::end);
	temparam.read((char*) &tempf, 4);
	newparam<<tempf/10<<"    ;[nm]"<<endl;
	newparam<<"Field Y Size in nm               : ";
	temparam.seekg(-20,ios::end);
	temparam.read((char*) &tempf, 4);
	newparam<<tempf/10<<"    ;[nm]"<<endl;
	newparam<<"Image Size in X                  : "<<pnts<<endl;
	newparam<<"Image Size in Y                  : "<<lins<<endl;
	newparam<<"Increment X                      : ";
	temparam.seekg(-24,ios::end);
	temparam.read((char*) &tempf, 4);
	newparam<<tempf/10<<"    ;[nm]"<<endl;
	newparam<<"Increment Y                      : ";
	temparam.seekg(-28,ios::end);
	temparam.read((char*) &tempf, 4);
	newparam<<tempf/10<<"    ;[nm]"<<endl;
	newparam<<"Scan Angle                       : ";
	temparam.seekg(-40,ios::end);
	temparam.read((char*) &tempf, 4);
	newparam<<tempf<<"    ;[Degree]"<<endl;
	newparam<<"X Offset                         : 0    ;[nm] (made up)"<<endl;
	newparam<<"Y Offset                         : 0    ;[nm] (made up)"<<endl<<endl;
	newparam<<";"<<endl;
	newparam<<"; Topographic Channels."<<endl;
	newparam<<";"<<endl<<endl;
	newparam<<"Topographic Channel              : Z    ;Channel"<<endl;
	newparam<<"                                   Forward"<<endl;
	newparam<<"                                   -32767    ;Minimum raw value"<<endl;
	newparam<<"                                   32766    ;Maximum raw value"<<endl;
	temparam.seekg(-52,ios::end);
	temparam.read((char*) &tempf, 4);
	newparam<<"                                   ";
	newparam<<-32767*tempf/10<<"    ;Minimum value in physical unit"<<endl;
	newparam<<"                                   ";
	newparam<<32766*tempf/10<<"    ;Maximum value in physical unit"<<endl;
	newparam<<"                                   ";
	newparam<<tempf/10<<"    ;Resolution"<<endl;
	newparam<<"                                   nm    ;Physical unit"<<endl;
	newparam<<"                                   m";
	newparam<<fnum<<"_ori.tf0    ;Filename"<<endl;
	newparam<<"                                   Z    ;Display name"<<endl<<endl;
	if (backflag)
		{
		newparam<<"Topographic Channel              : Z    ;Channel"<<endl;
		newparam<<"                                   Backward"<<endl;
		newparam<<"                                   -32767    ;Minimum raw value"<<endl;
		newparam<<"                                   32766    ;Maximum raw value"<<endl;
		temparam.seekg(-52,ios::end);
		temparam.read((char*) &tempf, 4);
		newparam<<"                                   ";
		tempf/=10;
		newparam<<-32767*tempf<<"    ;Minimum value in physical unit"<<endl;
		newparam<<"                                   ";
		newparam<<32766*tempf<<"    ;Maximum value in physical unit"<<endl;
		newparam<<"                                   ";
		newparam<<tempf<<"    ;Resolution"<<endl;
		newparam<<"                                   nm    ;Physical unit"<<endl;
		newparam<<"                                   m";
		newparam<<fnum<<"_ori.tb0    ;Filename"<<endl;
		newparam<<"                                   Z    ;Display name"<<endl<<endl;
		}

	if (specflag)
		{
		newparam<<";"<<endl;
		newparam<<"; Spectroscopy Channels."<<endl;
		newparam<<";"<<endl<<endl;
		newparam<<"Spectroscopy Channel             : I    ;Channel"<<endl;
		newparam<<"                                   V    ;Parameter"<<endl;
		newparam<<"                                   Forward"<<endl;
		newparam<<"                                   0    ;Minimum raw value"<<endl;
		newparam<<"                                   32768    ;Maximum raw value"<<endl;
		newparam<<"                                   0.00000    ;Minimum value in physical units"<<endl;
		newparam<<"                                   50.000    ;Maximum value in physical units"<<endl;
		newparam<<"                                   0.0015259    ;resolution"<<endl;
		newparam<<"                                   nA    ;Physical unit"<<endl;
		newparam<<"                                   ";
		temparam.seekg(-168,ios::end);
		temparam.read((char*) &tempi, 4);
		newparam<<tempi<<"    ;Number of spectroscopy points"<<endl;
		newparam<<"                                   ";
		temparam.seekg(-156,ios::end);
		temparam.read((char*) &tempf, 4);
		newparam<<tempf<<"    ;Start point spectroscopy"<<endl;
		newparam<<"                                   ";
		temparam.seekg(-160,ios::end);
		temparam.read((char*) &tempf, 4);
		newparam<<tempf<<"    ;End point spectroscopy"<<endl;
		newparam<<"                                   ";
		temparam.seekg(-164,ios::end);
		temparam.read((char*) &tempf, 4);
		newparam<<tempf<<"    ;Increment point spectroscopy"<<endl;
		newparam<<"                                   ";
		temparam.seekg(-156,ios::end);
		temparam.read((char*) &tempi, 4);
		newparam<<(float) tempi/2<<"    ;Acquisition time per spectroscopy point"<<endl;
		newparam<<"                                   ";
		newparam<<(float) tempi/2<<"    ;delay time per spectroscopy point"<<endl;
		newparam<<"                                    Off   ;Feeedback on/off"<<endl;
		newparam<<"                                    m";
		newparam<<fnum<<"_ori.sf0    ;Filename"<<endl;
		newparam<<"                                    I    ;Display name"<<endl<<endl;
		if (backflag)
			{
			newparam<<"Spectroscopy Channel             : I    ;Channel"<<endl;
    		newparam<<"                                   V    ;Parameter"<<endl;
    		newparam<<"                                   Backward"<<endl;
    		newparam<<"                                   0    ;Minimum raw value"<<endl;
    		newparam<<"                                   32768    ;Maximum raw value"<<endl;
    		newparam<<"                                   0.00000    ;Minimum value in physical units"<<endl;
    		newparam<<"                                   50.000    ;Maximum value in physical units"<<endl;
    		newparam<<"                                   0.0015259    ;resolution"<<endl;
			newparam<<"                                   nA    ;Physical unit"<<endl;
    		newparam<<"                                   ";
    		temparam.seekg(-184,ios::end);
    		temparam.read((char*) &tempi, 4);
    		newparam<<tempi<<"    ;Number of spectroscopy points"<<endl;
    		newparam<<"                                   ";
    		temparam.seekg(-172,ios::end);
    		temparam.read((char*) &tempf, 4);
    		newparam<<tempf<<"    ;Start point spectroscopy"<<endl;
    		newparam<<"                                   ";
    		temparam.seekg(-176,ios::end);
    		temparam.read((char*) &tempf, 4);
    		newparam<<tempf<<"    ;End point spectroscopy"<<endl;
    		newparam<<"                                   ";
    		temparam.seekg(-180,ios::end);
    		temparam.read((char*) &tempf, 4);
    		newparam<<tempf<<"    ;Increment point spectroscopy"<<endl;
    		newparam<<"                                   ";
    		temparam.seekg(-156,ios::end);
    		temparam.read((char*) &tempi, 4);
    		newparam<<(float) tempi/2<<"    ;Acquisition time per spectroscopy point"<<endl;
    		newparam<<"                                   ";
    		newparam<<(float) tempi/2<<"    ;delay time per spectroscopy point"<<endl;
    		newparam<<"                                    Off   ;Feeedback on/off"<<endl;
    		newparam<<"                                    m";
    		newparam<<fnum<<"_ori.sb0    ;Filename"<<endl;
    		newparam<<"                                    I    ;Display name"<<endl<<endl;
			}
		}
	newparam<<";"<<endl;
	newparam<<"; Measurement parameters."<<endl;
	newparam<<";"<<endl<<endl;
	newparam<<"SPM Method                           : STM"<<endl;
	if (backflag)
		{
		newparam<<"Dual mode                            : On"<<endl;
		newparam<<"Delay                                : 0    ;[us]  (Forward)"<<endl;
		newparam<<"                                     : 0    ;[us]  (Backward)"<<endl;
		}
	else
		newparam<<"Dual mode                            : Off"<<endl;

	newparam<<"Gap Voltage                          : ";
	/* ******Back and forward ** */
	temparam.seekg(-120,ios::end);
	temparam.read((char*) &tempf, 4);
	newparam<<tempf<<"    ;[V]";
	if (backflag)
		{
		newparam<<"  (Forward)"<<endl;
		newparam<<"                                       ";
		temparam.seekg(-124,ios::end);
		temparam.read((char*) &tempf, 4);
		newparam<<tempf<<"    ;[V]  (Backward)"<<endl;
		}
	else
		newparam<<endl;
	newparam<<"Feedback Set                         : ";
	/* ******back and forward *** */
	temparam.seekg(-140,ios::end);
	temparam.read((char*) &tempf, 4);
	newparam<<tempf<<"    ;[nA]";
	if (backflag)
		{
		newparam<<"  (Forward)"<<endl;
		newparam<<"                                       ";
		temparam.seekg(-144,ios::end);
		temparam.read((char*) &tempf, 4);
		newparam<<tempf<<"    ;[nA]  (Backward)"<<endl;
		}
	else
		newparam<<endl;
	newparam<<"Loop Gain                            : 1.2    ;[%] (made up)";
	if (backflag)
		{
		newparam<<"  (Forward)"<<endl;
		newparam<<"                                       1.2    ;[%] (made up)  (Backward)"<<endl;
		}
	else
		newparam<<endl;
	/* ***back and forward  ** */
	newparam<<"X Resolution                         : 0    ;Currently not used."<<endl;
	newparam<<"Y Resolution                         : 0    ;Currently not used."<<endl;
	newparam<<"Measured Gap Voltage                 : ";
	temparam.seekg(-116,ios::end);
	temparam.read((char*) &tempf, 4);
	newparam<<tempf<<"    ;[V] (made up)"<<endl;
	newparam<<"Measured Current Setpoint            : ";
	temparam.seekg(-136,ios::end);
	temparam.read((char*) &tempf, 4);
	newparam<<tempf<<"   ;[nA] (made up)"<<endl;
	newparam<<"Measured Current                     : ";
	newparam<<tempf<<"   ;[nA] (made up)"<<endl;
	newparam<<"Scan Speed                           : ";
	temparam.seekg(-252,ios::end);
	temparam.read((char*) &tempi, 4);
	temp2=(float) tempi* (float) pnts;
	temp2=1/temp2;
	temparam.seekg(-16,ios::end);
	temparam.read((char*) &tempf, 4);
	temp2*=tempf;
	temp2/=10;
	newparam<<temp2<<"    ;[nm/s]"<<endl;
	newparam<<"X Drift                              : 0    ;[nm/s] (made up)"<<endl;
	newparam<<"Y Drift                              : 0    ;[nm/s] (made up)"<<endl;
	newparam<<"Scan Mode                            : Frame"<<endl;
	newparam<<"Scan Direction                       : UP"<<endl;
	newparam<<"Spectroscopy Grid Value in X         : 1"<<endl;
	newparam<<"Spectroscopy Grid Value in Y         : 1"<<endl;
	newparam<<"Spectroscopy Points in X             : "<<pnts<<endl;
	newparam<<"Spectroscopy Lines in Y              : "<<lins<<endl<<endl;
	newparam<<";"<<endl;
	newparam<<"; Z Control Parameters."<<endl;
	newparam<<";"<<endl<<endl;
	newparam<<"Z Speed                              : ";
	temparam.seekg(-112,ios::end);
	temparam.read((char*) &tempi, 4);
	tempf=(float) tempi;
	newparam<<tempf/10<<"    ;[nm/s]"<<endl;
	newparam<<"Z Output Gain                        : 0.1    ;(made up)"<<endl;
	newparam<<"Automatic Z zero                     : On    ;(made up)"<<endl;
	newparam<<"Z Input Gain                         : 10    ;(made up)"<<endl<<endl;
	newparam<<";"<<endl;
	newparam<<"; AFM Description."<<endl;
	newparam<<";"<<endl<<endl;
	newparam<<"AFM exists                           : 1"<<endl;
	newparam<<"Amplitude                            : 0.1    ;[V]"<<endl;
	newparam<<"Delta f sign                         : -1"<<endl;
	newparam<<"Oscillator available                 : Yes"<<endl;
	newparam<<"Oscillator frequency                 : 455300    ;[Hz]"<<endl;
	newparam<<"Calibration FN                       : 10    ;[nN/V]"<<endl;
	newparam<<"Calibration FL                       : 10    ;[nN/V]"<<endl;
	newparam<<"I Total                              : 0.01    ;[V]"<<endl;

	newparam.close();
	temparam.close();
#ifdef DEBUG
	cout<<"Debug flag 1"<<endl;
#endif
	oldtopog.close();
	oldparam.close();
	newtopog.close();
	if (specflag)
		{
		oldspec.close();
		}
	if (remove("tmp.par"))
		{
		cout<<"Warning ! Could not delete temperary parameter file !"<<endl;
		cout<<"Continuing.."<<endl;
		}
	cout<<"File number "<<fnum<<" Done!"<<endl;


#ifdef DEBUG
if (oldparam.is_open())
	cout<<"Old param open"<<endl;
if (newparam.is_open())
	cout<<"New param open"<<endl;
if (oldspec.is_open())
	cout<<"Old spec open"<<endl;
if (newspec.is_open())
	cout<<"New spec open"<<endl;
if (oldtopog.is_open())
	cout<<"Old topog open"<<endl;
if (newtopog.is_open())
	cout<<"New topog open"<<endl;
if (temparam.is_open())
	cout<<"Temp param open"<<endl;
if (debugstrm.is_open())
	cout<<"Debug stream open"<<endl;
#endif


return 0;
}


int uns2s16bit(char highbyte, char lowbyte)
{
int value,i;
i=highbyte;
if (i<0)
	{
	i&=127;
	i+=128;
	}
value=i*256;
i=lowbyte;
if (i<0)
	{
	i&=127;
	i+=128;
	}
value+=i;
return value;
}


void errormess(char* fnum, int ernum)
{
cout<<"Error converting file number "<<fnum<<endl;
#ifdef DEBUG
if (i==100)
	{
	cout<<"Error 100: Couldn't open the debugging file!!"<<endl;
	cout<<"Exiting.."<<endl<<endl;
	return;
	}
#endif
if (ernum==9)
	{
	cout<<"Error 9: General error"<<endl;
	cout<<"Exiting.."<<endl<<endl;
	return;
	}
cout<<"Error "<<ernum<<": Couldn't open ";
switch (ernum)
	{
	case 1:
		{
		cout<<"old parameter file"<<endl;
		break;
		}
	case 2:
		{
		cout<<"old topography file"<<endl;
		break;
		}
	case 3:
		{
		cout<<"temp file"<<endl;
		break;
		}
	case 4:
		{
		cout<<"new parameter file"<<endl;
		break;
		}
	case 5:
		{
		cout<<"new topography file (forward)"<<endl;
		break;
		}
	case 6:
		{
		cout<<"new spectroscopy file (forward)"<<endl;
		break;
		}
	case 7:
		{
		cout<<"new topography file (backward)"<<endl;
		break;
		}
	case 8:
		{
		cout<<"new spectroscopy file (backward)"<<endl;
		break;
		}
	}
cout<<"Exiting.."<<endl<<endl;
}